from ..base import AdyenServiceBase


class HostedOnboardingApi(AdyenServiceBase):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, client=None):
        super().__init__(client=client)
        self.service = "legalEntityManagement"
        self.baseUrl = "https://kyc-test.adyen.com/lem/v4"

    def get_link_to_adyenhosted_onboarding_page(self, request, id, idempotency_key=None, **kwargs):
        """
        Get a link to an Adyen-hosted onboarding page
        """
        endpoint = self.baseUrl + f"/legalEntities/{id}/onboardingLinks"
        method = "POST"
        return self.client.call_adyen_api(
            request, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def get_onboarding_link_theme(self, id, idempotency_key=None, **kwargs):
        """
        Get an onboarding link theme
        """
        endpoint = self.baseUrl + f"/themes/{id}"
        method = "GET"
        return self.client.call_adyen_api(
            None, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def list_hosted_onboarding_page_themes(self, idempotency_key=None, **kwargs):
        """
        Get a list of hosted onboarding page themes
        """
        endpoint = self.baseUrl + "/themes"
        method = "GET"
        return self.client.call_adyen_api(
            None, self.service, method, endpoint, idempotency_key, **kwargs
        )



from ..base import AdyenServiceBase


class SCAAssociationManagementApi(AdyenServiceBase):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, client=None):
        super().__init__(client=client)
        self.service = "balancePlatform"
        self.baseUrl = "https://balanceplatform-api-test.adyen.com/bcl/v2"

    def approve_association(self, request, idempotency_key=None, **kwargs):
        """
        Approve a pending approval association
        """
        endpoint = self.baseUrl + "/scaAssociations"
        method = "PATCH"
        return self.client.call_adyen_api(
            request, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def list_associations(self, idempotency_key=None, **kwargs):
        """
        Get a list of devices associated with an entity
        """
        endpoint = self.baseUrl + "/scaAssociations"
        method = "GET"
        return self.client.call_adyen_api(
            None, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def remove_association(self, request, idempotency_key=None, **kwargs):
        """
        Delete association to devices
        """
        endpoint = self.baseUrl + "/scaAssociations"
        method = "DELETE"
        return self.client.call_adyen_api(
            request, self.service, method, endpoint, idempotency_key, **kwargs
        )



from ..base import AdyenServiceBase


class DirectDebitMandatesApi(AdyenServiceBase):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, client=None):
        super().__init__(client=client)
        self.service = "balancePlatform"
        self.baseUrl = "https://balanceplatform-api-test.adyen.com/bcl/v2"

    def cancel_mandate(self, mandateId, idempotency_key=None, **kwargs):
        """
        Cancel a mandate
        """
        endpoint = self.baseUrl + f"/mandates/{mandateId}/cancel"
        method = "POST"
        return self.client.call_adyen_api(
            None, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def get_list_of_mandates(self, idempotency_key=None, **kwargs):
        """
        Get a list of mandates
        """
        endpoint = self.baseUrl + "/mandates"
        method = "GET"
        return self.client.call_adyen_api(
            None, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def get_mandate_by_id(self, mandateId, idempotency_key=None, **kwargs):
        """
        Get a specific mandate
        """
        endpoint = self.baseUrl + f"/mandates/{mandateId}"
        method = "GET"
        return self.client.call_adyen_api(
            None, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def update_mandate(self, request, mandateId, idempotency_key=None, **kwargs):
        """
        Amend a mandate
        """
        endpoint = self.baseUrl + f"/mandates/{mandateId}"
        method = "PATCH"
        return self.client.call_adyen_api(
            request, self.service, method, endpoint, idempotency_key, **kwargs
        )



from ..base import AdyenServiceBase
from .bin_lookup_api import BinLookupApi


class AdyenBinlookupApi(AdyenServiceBase):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, client=None):
        super().__init__(client=client)
        self.bin_lookup_api = BinLookupApi(client=client)



from ..base import AdyenServiceBase


class TermsOfServiceApi(AdyenServiceBase):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, client=None):
        super().__init__(client=client)
        self.service = "legalEntityManagement"
        self.baseUrl = "https://kyc-test.adyen.com/lem/v4"

    def accept_terms_of_service(
        self, request, id, termsofservicedocumentid, idempotency_key=None, **kwargs
    ):
        """
        Accept Terms of Service
        """
        endpoint = self.baseUrl + f"/legalEntities/{id}/termsOfService/{termsofservicedocumentid}"
        method = "PATCH"
        return self.client.call_adyen_api(
            request, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def get_accepted_terms_of_service_document(
        self, id, termsofserviceacceptancereference, idempotency_key=None, **kwargs
    ):
        """
        Get accepted Terms of Service document
        """
        endpoint = (
            self.baseUrl
            + f"/legalEntities/{id}/acceptedTermsOfServiceDocument/{termsofserviceacceptancereference}"
        )
        method = "GET"
        return self.client.call_adyen_api(
            None, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def get_terms_of_service_document(self, request, id, idempotency_key=None, **kwargs):
        """
        Get Terms of Service document
        """
        endpoint = self.baseUrl + f"/legalEntities/{id}/termsOfService"
        method = "POST"
        return self.client.call_adyen_api(
            request, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def get_terms_of_service_information_for_legal_entity(self, id, idempotency_key=None, **kwargs):
        """
        Get Terms of Service information for a legal entity
        """
        endpoint = self.baseUrl + f"/legalEntities/{id}/termsOfServiceAcceptanceInfos"
        method = "GET"
        return self.client.call_adyen_api(
            None, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def get_terms_of_service_status(self, id, idempotency_key=None, **kwargs):
        """
        Get Terms of Service status
        """
        endpoint = self.baseUrl + f"/legalEntities/{id}/termsOfServiceStatus"
        method = "GET"
        return self.client.call_adyen_api(
            None, self.service, method, endpoint, idempotency_key, **kwargs
        )



from ..base import AdyenServiceBase


class APICredentialsMerchantLevelApi(AdyenServiceBase):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, client=None):
        super().__init__(client=client)
        self.service = "management"
        self.baseUrl = "https://management-test.adyen.com/v3"

    def create_api_credential(self, request, merchantId, idempotency_key=None, **kwargs):
        """
        Create an API credential
        """
        endpoint = self.baseUrl + f"/merchants/{merchantId}/apiCredentials"
        method = "POST"
        return self.client.call_adyen_api(
            request, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def get_api_credential(self, merchantId, apiCredentialId, idempotency_key=None, **kwargs):
        """
        Get an API credential
        """
        endpoint = self.baseUrl + f"/merchants/{merchantId}/apiCredentials/{apiCredentialId}"
        method = "GET"
        return self.client.call_adyen_api(
            None, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def list_api_credentials(self, merchantId, idempotency_key=None, **kwargs):
        """
        Get a list of API credentials
        """
        endpoint = self.baseUrl + f"/merchants/{merchantId}/apiCredentials"
        method = "GET"
        return self.client.call_adyen_api(
            None, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def update_api_credential(
        self, request, merchantId, apiCredentialId, idempotency_key=None, **kwargs
    ):
        """
        Update an API credential
        """
        endpoint = self.baseUrl + f"/merchants/{merchantId}/apiCredentials/{apiCredentialId}"
        method = "PATCH"
        return self.client.call_adyen_api(
            request, self.service, method, endpoint, idempotency_key, **kwargs
        )



from ..base import AdyenServiceBase


class ManageSCADevicesApi(AdyenServiceBase):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, client=None):
        super().__init__(client=client)
        self.service = "balancePlatform"
        self.baseUrl = "https://balanceplatform-api-test.adyen.com/bcl/v2"

    def complete_association_between_sca_device_and_resource(
        self, request, deviceId, idempotency_key=None, **kwargs
    ):
        """
        Complete an association between an SCA device and a resource
        """
        endpoint = self.baseUrl + f"/registeredDevices/{deviceId}/associations"
        method = "PATCH"
        return self.client.call_adyen_api(
            request, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def complete_registration_of_sca_device(self, request, id, idempotency_key=None, **kwargs):
        """
        Complete the registration of an SCA device
        """
        endpoint = self.baseUrl + f"/registeredDevices/{id}"
        method = "PATCH"
        return self.client.call_adyen_api(
            request, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def delete_registration_of_sca_device(self, id, idempotency_key=None, **kwargs):
        """
        Delete a registration of an SCA device
        """
        endpoint = self.baseUrl + f"/registeredDevices/{id}"
        method = "DELETE"
        return self.client.call_adyen_api(
            None, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def initiate_association_between_sca_device_and_resource(
        self, request, deviceId, idempotency_key=None, **kwargs
    ):
        """
        Initiate an association between an SCA device and a resource
        """
        endpoint = self.baseUrl + f"/registeredDevices/{deviceId}/associations"
        method = "POST"
        return self.client.call_adyen_api(
            request, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def initiate_registration_of_sca_device(self, request, idempotency_key=None, **kwargs):
        """
        Initiate the registration of an SCA device
        """
        endpoint = self.baseUrl + "/registeredDevices"
        method = "POST"
        return self.client.call_adyen_api(
            request, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def list_registered_sca_devices(self, idempotency_key=None, **kwargs):
        """
        Get a list of registered SCA devices
        """
        endpoint = self.baseUrl + "/registeredDevices"
        method = "GET"
        return self.client.call_adyen_api(
            None, self.service, method, endpoint, idempotency_key, **kwargs
        )



from ..base import AdyenServiceBase


class AuthorizedCardUsersApi(AdyenServiceBase):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, client=None):
        super().__init__(client=client)
        self.service = "balancePlatform"
        self.baseUrl = "https://balanceplatform-api-test.adyen.com/bcl/v2"

    def create_authorised_card_users(
        self, request, paymentInstrumentId, idempotency_key=None, **kwargs
    ):
        """
        Create authorized users for a card.
        """
        endpoint = self.baseUrl + f"/paymentInstruments/{paymentInstrumentId}/authorisedCardUsers"
        method = "POST"
        return self.client.call_adyen_api(
            request, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def delete_authorised_card_users(self, paymentInstrumentId, idempotency_key=None, **kwargs):
        """
        Delete the authorized users for a card.
        """
        endpoint = self.baseUrl + f"/paymentInstruments/{paymentInstrumentId}/authorisedCardUsers"
        method = "DELETE"
        return self.client.call_adyen_api(
            None, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def get_all_authorised_card_users(self, paymentInstrumentId, idempotency_key=None, **kwargs):
        """
        Get authorized users for a card.
        """
        endpoint = self.baseUrl + f"/paymentInstruments/{paymentInstrumentId}/authorisedCardUsers"
        method = "GET"
        return self.client.call_adyen_api(
            None, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def update_authorised_card_users(
        self, request, paymentInstrumentId, idempotency_key=None, **kwargs
    ):
        """
        Update the authorized users for a card.
        """
        endpoint = self.baseUrl + f"/paymentInstruments/{paymentInstrumentId}/authorisedCardUsers"
        method = "PATCH"
        return self.client.call_adyen_api(
            request, self.service, method, endpoint, idempotency_key, **kwargs
        )



from ..base import AdyenServiceBase


class SCADeviceManagementApi(AdyenServiceBase):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, client=None):
        super().__init__(client=client)
        self.service = "balancePlatform"
        self.baseUrl = "https://balanceplatform-api-test.adyen.com/bcl/v2"

    def begin_sca_device_registration(self, request, idempotency_key=None, **kwargs):
        """
        Begin SCA device registration
        """
        endpoint = self.baseUrl + "/scaDevices"
        method = "POST"
        return self.client.call_adyen_api(
            request, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def finish_sca_device_registration(self, request, deviceId, idempotency_key=None, **kwargs):
        """
        Finish registration process for a SCA device
        """
        endpoint = self.baseUrl + f"/scaDevices/{deviceId}"
        method = "PATCH"
        return self.client.call_adyen_api(
            request, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def submit_sca_association(self, request, deviceId, idempotency_key=None, **kwargs):
        """
        Create a new SCA association for a device
        """
        endpoint = self.baseUrl + f"/scaDevices/{deviceId}/scaAssociations"
        method = "POST"
        return self.client.call_adyen_api(
            request, self.service, method, endpoint, idempotency_key, **kwargs
        )



class AdyenError(Exception):
    def __init__(
        self,
        message,
        raw_request="",
        raw_response="",
        url="",
        psp="",
        headers="",
        status_code="",
        error_code="",
    ):
        self.message = message
        self.raw_request = raw_request
        self.raw_response = raw_response
        self.url = url
        self.psp = psp
        self.headers = headers
        self.status_code = status_code
        self.error_code = error_code

    def __str__(self):
        return f"{self.__class__.__name__}:{self.message}"

    def debug(self):
        return (
            f"class: {self.__class__.__name__}\nmessage: {self.message}\nHTTP status_code:{self.status_code}\nerror_code:{self.error_code}\nurl: {self.url}\n"
            f"request: {self.raw_request}\nresponse: {self.raw_response}\nheaders: {self.headers}"
        )


class AdyenInvalidRequestError(AdyenError):
    pass


class AdyenAPIResponseError(AdyenError):
    def __init__(self, message, *args, **kwargs):
        super().__init__(message, *args, **kwargs)


class AdyenAPIAuthenticationError(AdyenAPIResponseError):
    pass


class AdyenAPIInvalidPermission(AdyenAPIResponseError):
    pass


class AdyenAPICommunicationError(AdyenAPIResponseError):
    pass


class AdyenAPIValidationError(AdyenAPIResponseError):
    pass


class AdyenAPIUnprocessableEntity(AdyenAPIResponseError):
    pass


class AdyenAPIInvalidFormat(AdyenAPIResponseError):
    pass


class AdyenEndpointInvalidFormat(AdyenError):
    pass



from .balancePlatform import AdyenBalancePlatformApi as AdyenBalancePlatformApi
from .base import AdyenBase as AdyenBase
from .binlookup import AdyenBinlookupApi as AdyenBinlookupApi
from .capital import AdyenCapitalApi as AdyenCapitalApi
from .checkout import AdyenCheckoutApi as AdyenCheckoutApi
from .dataProtection import AdyenDataProtectionApi as AdyenDataProtectionApi
from .disputes import AdyenDisputesApi as AdyenDisputesApi
from .legalEntityManagement import AdyenLegalEntityManagementApi as AdyenLegalEntityManagementApi
from .management import AdyenManagementApi as AdyenManagementApi
from .payments import AdyenPaymentsApi as AdyenPaymentsApi
from .payouts import AdyenPayoutsApi as AdyenPayoutsApi
from .posMobile import AdyenPosMobileApi as AdyenPosMobileApi
from .recurring import AdyenRecurringApi as AdyenRecurringApi
from .sessionAuthentication import AdyenSessionAuthenticationApi as AdyenSessionAuthenticationApi
from .storedValue import AdyenStoredValueApi as AdyenStoredValueApi
from .terminal import AdyenTerminalApi as AdyenTerminalApi
from .transfers import AdyenTransfersApi as AdyenTransfersApi



from ..base import AdyenServiceBase
from .session_authentication_api import SessionAuthenticationApi


class AdyenSessionAuthenticationApi(AdyenServiceBase):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, client=None):
        super().__init__(client=client)
        self.session_authentication_api = SessionAuthenticationApi(client=client)



from ..base import AdyenServiceBase


class RecurringApi(AdyenServiceBase):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, client=None):
        super().__init__(client=client)
        self.service = "recurring"
        self.baseUrl = (
            "https://paltokenization-test.adyen.com/paltokenization/servlet/Recurring/v68"
        )

    def create_permit(self, request, idempotency_key=None, **kwargs):
        """
        Create new permits linked to a recurring contract.

        Deprecated since Adyen Recurring API v68
        """
        endpoint = self.baseUrl + "/createPermit"
        method = "POST"
        return self.client.call_adyen_api(
            request, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def disable(self, request, idempotency_key=None, **kwargs):
        """
        Disable stored payment details
        """
        endpoint = self.baseUrl + "/disable"
        method = "POST"
        return self.client.call_adyen_api(
            request, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def disable_permit(self, request, idempotency_key=None, **kwargs):
        """
        Disable an existing permit.

        Deprecated since Adyen Recurring API v68
        """
        endpoint = self.baseUrl + "/disablePermit"
        method = "POST"
        return self.client.call_adyen_api(
            request, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def list_recurring_details(self, request, idempotency_key=None, **kwargs):
        """
        Get stored payment details
        """
        endpoint = self.baseUrl + "/listRecurringDetails"
        method = "POST"
        return self.client.call_adyen_api(
            request, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def notify_shopper(self, request, idempotency_key=None, **kwargs):
        """
        Ask issuer to notify the shopper
        """
        endpoint = self.baseUrl + "/notifyShopper"
        method = "POST"
        return self.client.call_adyen_api(
            request, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def schedule_account_updater(self, request, idempotency_key=None, **kwargs):
        """
        Schedule running the Account Updater
        """
        endpoint = self.baseUrl + "/scheduleAccountUpdater"
        method = "POST"
        return self.client.call_adyen_api(
            request, self.service, method, endpoint, idempotency_key, **kwargs
        )



from ..base import AdyenServiceBase


class OrdersApi(AdyenServiceBase):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, client=None):
        super().__init__(client=client)
        self.service = "checkout"
        self.baseUrl = "https://checkout-test.adyen.com/v71"

    def cancel_order(self, request, idempotency_key=None, **kwargs):
        """
        Cancel an order
        """
        endpoint = self.baseUrl + "/orders/cancel"
        method = "POST"
        return self.client.call_adyen_api(
            request, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def get_balance_of_gift_card(self, request, idempotency_key=None, **kwargs):
        """
        Get the balance of a gift card
        """
        endpoint = self.baseUrl + "/paymentMethods/balance"
        method = "POST"
        return self.client.call_adyen_api(
            request, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def orders(self, request, idempotency_key=None, **kwargs):
        """
        Create an order
        """
        endpoint = self.baseUrl + "/orders"
        method = "POST"
        return self.client.call_adyen_api(
            request, self.service, method, endpoint, idempotency_key, **kwargs
        )



from Adyen import AdyenClient


class AdyenBase:
    def __setattr__(self, attr, value):
        client_attr = ["username", "password", "platform"]
        if attr in client_attr:
            if value:
                self.client[attr] = value
        else:
            super().__setattr__(attr, value)

    def __getattr__(self, attr):
        client_attr = ["username", "password", "platform"]
        if attr in client_attr:
            return self.client[attr]


class AdyenServiceBase(AdyenBase):
    def __init__(self, client=None):
        if client:
            self.client = client
        else:
            self.client = AdyenClient()



from ..base import AdyenServiceBase


class GrantOffersApi(AdyenServiceBase):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, client=None):
        super().__init__(client=client)
        self.service = "balancePlatform"
        self.baseUrl = "https://balanceplatform-api-test.adyen.com/bcl/v2"

    def get_all_available_grant_offers(self, idempotency_key=None, **kwargs):
        """
        Get all available grant offers

        Deprecated since Configuration API v2
        Use the `/grantOffers` endpoint from the [Capital API](https://docs.adyen.com/api-explorer/capital/latest/get/grantOffers) instead.
        """
        endpoint = self.baseUrl + "/grantOffers"
        method = "GET"
        return self.client.call_adyen_api(
            None, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def get_grant_offer(self, grantOfferId, idempotency_key=None, **kwargs):
        """
        Get a grant offer

        Deprecated since Configuration API v2
        Use the `/grantOffers/{id}` endpoint from the [Capital API](https://docs.adyen.com/api-explorer/capital/latest/get/grantOffers/(id)) instead.
        """
        endpoint = self.baseUrl + f"/grantOffers/{grantOfferId}"
        method = "GET"
        return self.client.call_adyen_api(
            None, self.service, method, endpoint, idempotency_key, **kwargs
        )



from ..base import AdyenServiceBase


class BalancesApi(AdyenServiceBase):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, client=None):
        super().__init__(client=client)
        self.service = "balancePlatform"
        self.baseUrl = "https://balanceplatform-api-test.adyen.com/bcl/v2"

    def create_webhook_setting(
        self, request, balancePlatformId, webhookId, idempotency_key=None, **kwargs
    ):
        """
        Create a balance webhook setting
        """
        endpoint = (
            self.baseUrl + f"/balancePlatforms/{balancePlatformId}/webhooks/{webhookId}/settings"
        )
        method = "POST"
        return self.client.call_adyen_api(
            request, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def delete_webhook_setting(
        self, balancePlatformId, webhookId, settingId, idempotency_key=None, **kwargs
    ):
        """
        Delete a balance webhook setting by id
        """
        endpoint = (
            self.baseUrl
            + f"/balancePlatforms/{balancePlatformId}/webhooks/{webhookId}/settings/{settingId}"
        )
        method = "DELETE"
        return self.client.call_adyen_api(
            None, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def get_all_webhook_settings(
        self, balancePlatformId, webhookId, idempotency_key=None, **kwargs
    ):
        """
        Get all balance webhook settings
        """
        endpoint = (
            self.baseUrl + f"/balancePlatforms/{balancePlatformId}/webhooks/{webhookId}/settings"
        )
        method = "GET"
        return self.client.call_adyen_api(
            None, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def get_webhook_setting(
        self, balancePlatformId, webhookId, settingId, idempotency_key=None, **kwargs
    ):
        """
        Get a balance webhook setting by id
        """
        endpoint = (
            self.baseUrl
            + f"/balancePlatforms/{balancePlatformId}/webhooks/{webhookId}/settings/{settingId}"
        )
        method = "GET"
        return self.client.call_adyen_api(
            None, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def update_webhook_setting(
        self, request, balancePlatformId, webhookId, settingId, idempotency_key=None, **kwargs
    ):
        """
        Update a balance webhook setting by id
        """
        endpoint = (
            self.baseUrl
            + f"/balancePlatforms/{balancePlatformId}/webhooks/{webhookId}/settings/{settingId}"
        )
        method = "PATCH"
        return self.client.call_adyen_api(
            request, self.service, method, endpoint, idempotency_key, **kwargs
        )



from ..base import AdyenServiceBase


class UsersCompanyLevelApi(AdyenServiceBase):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, client=None):
        super().__init__(client=client)
        self.service = "management"
        self.baseUrl = "https://management-test.adyen.com/v3"

    def create_new_user(self, request, companyId, idempotency_key=None, **kwargs):
        """
        Create a new user
        """
        endpoint = self.baseUrl + f"/companies/{companyId}/users"
        method = "POST"
        return self.client.call_adyen_api(
            request, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def get_user_details(self, companyId, userId, idempotency_key=None, **kwargs):
        """
        Get user details
        """
        endpoint = self.baseUrl + f"/companies/{companyId}/users/{userId}"
        method = "GET"
        return self.client.call_adyen_api(
            None, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def list_users(self, companyId, idempotency_key=None, **kwargs):
        """
        Get a list of users
        """
        endpoint = self.baseUrl + f"/companies/{companyId}/users"
        method = "GET"
        return self.client.call_adyen_api(
            None, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def update_user_details(self, request, companyId, userId, idempotency_key=None, **kwargs):
        """
        Update user details
        """
        endpoint = self.baseUrl + f"/companies/{companyId}/users/{userId}"
        method = "PATCH"
        return self.client.call_adyen_api(
            request, self.service, method, endpoint, idempotency_key, **kwargs
        )



from ..base import AdyenServiceBase


class DynamicOffersApi(AdyenServiceBase):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, client=None):
        super().__init__(client=client)
        self.service = "capital"
        self.baseUrl = "https://balanceplatform-api-test.adyen.com/capital/v1"

    def calculate_preliminary_offer_from_dynamic_offer(
        self, request, id, idempotency_key=None, **kwargs
    ):
        """
        Calculate a preliminary offer for a selected financing amount
        """
        endpoint = self.baseUrl + f"/dynamicOffers/{id}/calculate"
        method = "POST"
        return self.client.call_adyen_api(
            request, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def create_static_offer_from_dynamic_offer(self, request, id, idempotency_key=None, **kwargs):
        """
        Create a static offer for a selected financing amount
        """
        endpoint = self.baseUrl + f"/dynamicOffers/{id}/grantOffer"
        method = "POST"
        return self.client.call_adyen_api(
            request, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def get_all_dynamic_offers(self, idempotency_key=None, **kwargs):
        """
        Get all available dynamic offers
        """
        endpoint = self.baseUrl + "/dynamicOffers"
        method = "GET"
        return self.client.call_adyen_api(
            None, self.service, method, endpoint, idempotency_key, **kwargs
        )



from ..base import AdyenServiceBase


class TransfersApi(AdyenServiceBase):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, client=None):
        super().__init__(client=client)
        self.service = "transfers"
        self.baseUrl = "https://balanceplatform-api-test.adyen.com/btl/v4"

    def approve_initiated_transfers(self, request, idempotency_key=None, **kwargs):
        """
        Approve initiated transfers
        """
        endpoint = self.baseUrl + "/transfers/approve"
        method = "POST"
        return self.client.call_adyen_api(
            request, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def cancel_initiated_transfers(self, request, idempotency_key=None, **kwargs):
        """
        Cancel initiated transfers
        """
        endpoint = self.baseUrl + "/transfers/cancel"
        method = "POST"
        return self.client.call_adyen_api(
            request, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def get_all_transfers(self, idempotency_key=None, **kwargs):
        """
        Get all transfers
        """
        endpoint = self.baseUrl + "/transfers"
        method = "GET"
        return self.client.call_adyen_api(
            None, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def get_transfer(self, id, idempotency_key=None, **kwargs):
        """
        Get a transfer
        """
        endpoint = self.baseUrl + f"/transfers/{id}"
        method = "GET"
        return self.client.call_adyen_api(
            None, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def return_transfer(self, request, transferId, idempotency_key=None, **kwargs):
        """
        Return a transfer
        """
        endpoint = self.baseUrl + f"/transfers/{transferId}/returns"
        method = "POST"
        return self.client.call_adyen_api(
            request, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def transfer_funds(self, request, idempotency_key=None, **kwargs):
        """
        Transfer funds
        """
        endpoint = self.baseUrl + "/transfers"
        method = "POST"
        return self.client.call_adyen_api(
            request, self.service, method, endpoint, idempotency_key, **kwargs
        )



from . import util as util
from .client import AdyenClient
from .exceptions import (
    AdyenAPIAuthenticationError as AdyenAPIAuthenticationError,
)
from .exceptions import (
    AdyenAPICommunicationError as AdyenAPICommunicationError,
)
from .exceptions import (
    AdyenAPIInvalidPermission as AdyenAPIInvalidPermission,
)
from .exceptions import (
    AdyenAPIUnprocessableEntity as AdyenAPIUnprocessableEntity,
)
from .exceptions import (
    AdyenAPIValidationError as AdyenAPIValidationError,
)
from .exceptions import (
    AdyenError as AdyenError,
)
from .exceptions import (
    AdyenInvalidRequestError as AdyenInvalidRequestError,
)
from .httpclient import HTTPClient as HTTPClient
from .services import (
    AdyenBalancePlatformApi,
    AdyenBase,
    AdyenBinlookupApi,
    AdyenCapitalApi,
    AdyenCheckoutApi,
    AdyenDataProtectionApi,
    AdyenDisputesApi,
    AdyenLegalEntityManagementApi,
    AdyenManagementApi,
    AdyenPaymentsApi,
    AdyenPayoutsApi,
    AdyenPosMobileApi,
    AdyenRecurringApi,
    AdyenSessionAuthenticationApi,
    AdyenStoredValueApi,
    AdyenTerminalApi,
    AdyenTransfersApi,
)


class Adyen(AdyenBase):
    def __init__(self, **kwargs):
        self.client = AdyenClient(**kwargs)
        self.payment = AdyenPaymentsApi(client=self.client)
        self.capital = AdyenCapitalApi(client=self.client)
        self.binlookup = AdyenBinlookupApi(client=self.client)
        self.payout = AdyenPayoutsApi(client=self.client)
        self.recurring = AdyenRecurringApi(client=self.client)
        self.checkout = AdyenCheckoutApi(client=self.client)
        self.terminal = AdyenTerminalApi(client=self.client)
        self.management = AdyenManagementApi(client=self.client)
        self.legalEntityManagement = AdyenLegalEntityManagementApi(client=self.client)
        self.dataProtection = AdyenDataProtectionApi(client=self.client)
        self.transfers = AdyenTransfersApi(client=self.client)
        self.storedValue = AdyenStoredValueApi(client=self.client)
        self.balancePlatform = AdyenBalancePlatformApi(client=self.client)
        self.disputes = AdyenDisputesApi(client=self.client)
        self.sessionAuthentication = AdyenSessionAuthenticationApi(client=self.client)
        self.posMobile = AdyenPosMobileApi(client=self.client)


_base_adyen_obj = Adyen()
recurring = _base_adyen_obj.recurring
payment = _base_adyen_obj.payment
payout = _base_adyen_obj.payout
checkout = _base_adyen_obj.checkout
binlookup = _base_adyen_obj.binlookup
terminal = _base_adyen_obj.terminal
management = _base_adyen_obj.management
legalEntityManagement = _base_adyen_obj.legalEntityManagement
dataProtection = _base_adyen_obj.dataProtection
transfers = _base_adyen_obj.transfers
storedValue = _base_adyen_obj.storedValue
balancePlatform = _base_adyen_obj.balancePlatform
disputes = _base_adyen_obj.disputes
sessionAuthentication = _base_adyen_obj.sessionAuthentication
capital = _base_adyen_obj.capital
posMobile = _base_adyen_obj.posMobile



from ..base import AdyenServiceBase


class RecurringApi(AdyenServiceBase):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, client=None):
        super().__init__(client=client)
        self.service = "checkout"
        self.baseUrl = "https://checkout-test.adyen.com/v71"

    def delete_token_for_stored_payment_details(
        self, storedPaymentMethodId, idempotency_key=None, **kwargs
    ):
        """
        Delete a token for stored payment details
        """
        endpoint = self.baseUrl + f"/storedPaymentMethods/{storedPaymentMethodId}"
        method = "DELETE"
        return self.client.call_adyen_api(
            None, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def forward(self, request, idempotency_key=None, **kwargs):
        """
        Forward stored payment details
        """
        endpoint = self.baseUrl + "/forward"
        method = "POST"
        return self.client.call_adyen_api(
            request, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def get_tokens_for_stored_payment_details(self, idempotency_key=None, **kwargs):
        """
        Get tokens for stored payment details
        """
        endpoint = self.baseUrl + "/storedPaymentMethods"
        method = "GET"
        return self.client.call_adyen_api(
            None, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def stored_payment_methods(self, request, idempotency_key=None, **kwargs):
        """
        Create a token to store payment details
        """
        endpoint = self.baseUrl + "/storedPaymentMethods"
        method = "POST"
        return self.client.call_adyen_api(
            request, self.service, method, endpoint, idempotency_key, **kwargs
        )



import base64
import binascii
import copy
import hashlib
import hmac


# generates HMAC signature for the NotificationRequest object
def generate_notification_sig(dict_object, hmac_key):
    if not isinstance(dict_object, dict):
        raise ValueError("Must Provide dictionary object")

    hmac_key = binascii.a2b_hex(hmac_key)

    request_dict = dict(dict_object)
    request_dict["value"] = request_dict["amount"]["value"]
    request_dict["currency"] = request_dict["amount"]["currency"]

    element_orders = [
        "pspReference",
        "originalReference",
        "merchantAccountCode",
        "merchantReference",
        "value",
        "currency",
        "eventCode",
        "success",
    ]

    signing_string = ":".join(
        map(str, (request_dict.get(element, "") for element in element_orders))
    )

    hm = hmac.new(hmac_key, signing_string.encode("utf-8"), hashlib.sha256)
    return base64.b64encode(hm.digest())


# generates HMAC signature for the payload (bytes)
def generate_payload_sig(payload, hmac_key):
    if not isinstance(payload, bytes):
        raise ValueError("Must Provide payload as bytes")

    hmac_key = binascii.a2b_hex(hmac_key)

    hm = hmac.new(hmac_key, payload, hashlib.sha256)
    return base64.b64encode(hm.digest())


def is_valid_hmac_notification(dict_object, hmac_key):
    """
    validates the HMAC signature of the NotificationRequestItem object. Use for webhooks that provide the
    hmacSignature as part of the payload `AdditionalData` (i.e. Payments)
    Args:
        dict_object: object with a list of notificationItems
        hmac_key: HMAC key to generate the signature

    Returns:
        boolean: true when HMAC signature is valid
    """

    dict_object = copy.deepcopy(dict_object)

    if "notificationItems" in dict_object:
        dict_object = dict_object["notificationItems"][0]["NotificationRequestItem"]

    if "additionalData" in dict_object:
        if dict_object["additionalData"]["hmacSignature"] == "":
            raise ValueError("Must Provide hmacSignature in additionalData")
        else:
            expected_sign = dict_object["additionalData"]["hmacSignature"]
            del dict_object["additionalData"]
            merchant_sign = generate_notification_sig(dict_object, hmac_key)
            merchant_sign_str = merchant_sign.decode("utf-8")
            return hmac.compare_digest(merchant_sign_str, expected_sign)


def is_valid_hmac_payload(hmac_signature, hmac_key, payload):
    """
    validates the HMAC signature of a payload against an expected signature. Use for webhooks that provide the
    hmacSignature in the HTTP header (i.e. Banking, Management API)
    Args:
        hmac_signature: HMAC signature to validate
        hmac_key: HMAC key to generate the signature
        payload: webhook payload

    Returns:
        boolean: true when HMAC signature is valid
    """

    merchant_sign = generate_payload_sig(payload, hmac_key)
    merchant_sign_str = merchant_sign.decode("utf-8")

    return hmac.compare_digest(merchant_sign_str, hmac_signature)


def get_query(query_parameters):
    return "?" + "&".join([f"{k}={v}" for k, v in query_parameters.items()])



from ..base import AdyenServiceBase


class TaxEDeliveryConsentApi(AdyenServiceBase):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, client=None):
        super().__init__(client=client)
        self.service = "legalEntityManagement"
        self.baseUrl = "https://kyc-test.adyen.com/lem/v4"

    def check_status_of_consent_for_electronic_delivery_of_tax_forms(
        self, id, idempotency_key=None, **kwargs
    ):
        """
        Check the status of consent for electronic delivery of tax forms
        """
        endpoint = self.baseUrl + f"/legalEntities/{id}/checkTaxElectronicDeliveryConsent"
        method = "POST"
        return self.client.call_adyen_api(
            None, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def set_consent_status_for_electronic_delivery_of_tax_forms(
        self, request, id, idempotency_key=None, **kwargs
    ):
        """
        Set the consent status for electronic delivery of tax forms
        """
        endpoint = self.baseUrl + f"/legalEntities/{id}/setTaxElectronicDeliveryConsent"
        method = "POST"
        return self.client.call_adyen_api(
            request, self.service, method, endpoint, idempotency_key, **kwargs
        )



from ..base import AdyenServiceBase


class LegalEntitiesApi(AdyenServiceBase):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, client=None):
        super().__init__(client=client)
        self.service = "legalEntityManagement"
        self.baseUrl = "https://kyc-test.adyen.com/lem/v4"

    def check_legal_entitys_verification_errors(self, id, idempotency_key=None, **kwargs):
        """
        Check a legal entity's verification errors
        """
        endpoint = self.baseUrl + f"/legalEntities/{id}/checkVerificationErrors"
        method = "POST"
        return self.client.call_adyen_api(
            None, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def confirm_data_review(self, id, idempotency_key=None, **kwargs):
        """
        Confirm data review
        """
        endpoint = self.baseUrl + f"/legalEntities/{id}/confirmDataReview"
        method = "POST"
        return self.client.call_adyen_api(
            None, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def create_legal_entity(self, request, idempotency_key=None, **kwargs):
        """
        Create a legal entity
        """
        endpoint = self.baseUrl + "/legalEntities"
        method = "POST"
        return self.client.call_adyen_api(
            request, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def get_all_business_lines_under_legal_entity(self, id, idempotency_key=None, **kwargs):
        """
        Get all business lines under a legal entity
        """
        endpoint = self.baseUrl + f"/legalEntities/{id}/businessLines"
        method = "GET"
        return self.client.call_adyen_api(
            None, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def get_legal_entity(self, id, idempotency_key=None, **kwargs):
        """
        Get a legal entity
        """
        endpoint = self.baseUrl + f"/legalEntities/{id}"
        method = "GET"
        return self.client.call_adyen_api(
            None, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def request_periodic_review(self, id, idempotency_key=None, **kwargs):
        """
        Request periodic data review.
        """
        endpoint = self.baseUrl + f"/legalEntities/{id}/requestPeriodicReview"
        method = "POST"
        return self.client.call_adyen_api(
            None, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def update_legal_entity(self, request, id, idempotency_key=None, **kwargs):
        """
        Update a legal entity
        """
        endpoint = self.baseUrl + f"/legalEntities/{id}"
        method = "PATCH"
        return self.client.call_adyen_api(
            request, self.service, method, endpoint, idempotency_key, **kwargs
        )



from ..base import AdyenServiceBase
from .balance_control_api import BalanceControlApi


class AdyenBalanceControlApi(AdyenServiceBase):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, client=None):
        super().__init__(client=client)
        self.balance_control_api = BalanceControlApi(client=client)



from ..base import AdyenServiceBase


class TerminalActionsCompanyLevelApi(AdyenServiceBase):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, client=None):
        super().__init__(client=client)
        self.service = "management"
        self.baseUrl = "https://management-test.adyen.com/v3"

    def get_terminal_action(self, companyId, actionId, idempotency_key=None, **kwargs):
        """
        Get terminal action
        """
        endpoint = self.baseUrl + f"/companies/{companyId}/terminalActions/{actionId}"
        method = "GET"
        return self.client.call_adyen_api(
            None, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def list_terminal_actions(self, companyId, idempotency_key=None, **kwargs):
        """
        Get a list of terminal actions
        """
        endpoint = self.baseUrl + f"/companies/{companyId}/terminalActions"
        method = "GET"
        return self.client.call_adyen_api(
            None, self.service, method, endpoint, idempotency_key, **kwargs
        )



from ..base import AdyenServiceBase


class UtilityApi(AdyenServiceBase):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, client=None):
        super().__init__(client=client)
        self.service = "checkout"
        self.baseUrl = "https://checkout-test.adyen.com/v71"

    def get_apple_pay_session(self, request, idempotency_key=None, **kwargs):
        """
        Get an Apple Pay session
        """
        endpoint = self.baseUrl + "/applePay/sessions"
        method = "POST"
        return self.client.call_adyen_api(
            request, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def origin_keys(self, request, idempotency_key=None, **kwargs):
        """
        Create originKey values for domains

        Deprecated since Adyen Checkout API v67
        """
        endpoint = self.baseUrl + "/originKeys"
        method = "POST"
        return self.client.call_adyen_api(
            request, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def updates_order_for_paypal_express_checkout(self, request, idempotency_key=None, **kwargs):
        """
        Updates the order for PayPal Express Checkout
        """
        endpoint = self.baseUrl + "/paypal/updateOrder"
        method = "POST"
        return self.client.call_adyen_api(
            request, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def validate_shopper_id(self, request, idempotency_key=None, **kwargs):
        """
        Validates shopper Id
        """
        endpoint = self.baseUrl + "/validateShopperId"
        method = "POST"
        return self.client.call_adyen_api(
            request, self.service, method, endpoint, idempotency_key, **kwargs
        )



from ..base import AdyenServiceBase


class AccountStoreLevelApi(AdyenServiceBase):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, client=None):
        super().__init__(client=client)
        self.service = "management"
        self.baseUrl = "https://management-test.adyen.com/v3"

    def create_store(self, request, idempotency_key=None, **kwargs):
        """
        Create a store
        """
        endpoint = self.baseUrl + "/stores"
        method = "POST"
        return self.client.call_adyen_api(
            request, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def create_store_by_merchant_id(self, request, merchantId, idempotency_key=None, **kwargs):
        """
        Create a store
        """
        endpoint = self.baseUrl + f"/merchants/{merchantId}/stores"
        method = "POST"
        return self.client.call_adyen_api(
            request, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def get_store(self, merchantId, storeId, idempotency_key=None, **kwargs):
        """
        Get a store
        """
        endpoint = self.baseUrl + f"/merchants/{merchantId}/stores/{storeId}"
        method = "GET"
        return self.client.call_adyen_api(
            None, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def get_store_by_id(self, storeId, idempotency_key=None, **kwargs):
        """
        Get a store
        """
        endpoint = self.baseUrl + f"/stores/{storeId}"
        method = "GET"
        return self.client.call_adyen_api(
            None, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def list_stores(self, idempotency_key=None, **kwargs):
        """
        Get a list of stores
        """
        endpoint = self.baseUrl + "/stores"
        method = "GET"
        return self.client.call_adyen_api(
            None, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def list_stores_by_merchant_id(self, merchantId, idempotency_key=None, **kwargs):
        """
        Get a list of stores
        """
        endpoint = self.baseUrl + f"/merchants/{merchantId}/stores"
        method = "GET"
        return self.client.call_adyen_api(
            None, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def update_store(self, request, merchantId, storeId, idempotency_key=None, **kwargs):
        """
        Update a store
        """
        endpoint = self.baseUrl + f"/merchants/{merchantId}/stores/{storeId}"
        method = "PATCH"
        return self.client.call_adyen_api(
            request, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def update_store_by_id(self, request, storeId, idempotency_key=None, **kwargs):
        """
        Update a store
        """
        endpoint = self.baseUrl + f"/stores/{storeId}"
        method = "PATCH"
        return self.client.call_adyen_api(
            request, self.service, method, endpoint, idempotency_key, **kwargs
        )



from ..base import AdyenServiceBase


class BalanceAccountsApi(AdyenServiceBase):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, client=None):
        super().__init__(client=client)
        self.service = "balancePlatform"
        self.baseUrl = "https://balanceplatform-api-test.adyen.com/bcl/v2"

    def create_balance_account(self, request, idempotency_key=None, **kwargs):
        """
        Create a balance account
        """
        endpoint = self.baseUrl + "/balanceAccounts"
        method = "POST"
        return self.client.call_adyen_api(
            request, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def create_sweep(self, request, balanceAccountId, idempotency_key=None, **kwargs):
        """
        Create a sweep
        """
        endpoint = self.baseUrl + f"/balanceAccounts/{balanceAccountId}/sweeps"
        method = "POST"
        return self.client.call_adyen_api(
            request, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def delete_sweep(self, balanceAccountId, sweepId, idempotency_key=None, **kwargs):
        """
        Delete a sweep
        """
        endpoint = self.baseUrl + f"/balanceAccounts/{balanceAccountId}/sweeps/{sweepId}"
        method = "DELETE"
        return self.client.call_adyen_api(
            None, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def get_all_sweeps_for_balance_account(self, balanceAccountId, idempotency_key=None, **kwargs):
        """
        Get all sweeps for a balance account
        """
        endpoint = self.baseUrl + f"/balanceAccounts/{balanceAccountId}/sweeps"
        method = "GET"
        return self.client.call_adyen_api(
            None, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def get_all_transaction_rules_for_balance_account(self, id, idempotency_key=None, **kwargs):
        """
        Get all transaction rules for a balance account
        """
        endpoint = self.baseUrl + f"/balanceAccounts/{id}/transactionRules"
        method = "GET"
        return self.client.call_adyen_api(
            None, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def get_balance_account(self, id, idempotency_key=None, **kwargs):
        """
        Get a balance account
        """
        endpoint = self.baseUrl + f"/balanceAccounts/{id}"
        method = "GET"
        return self.client.call_adyen_api(
            None, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def get_payment_instruments_linked_to_balance_account(self, id, idempotency_key=None, **kwargs):
        """
        Get payment instruments linked to a balance account
        """
        endpoint = self.baseUrl + f"/balanceAccounts/{id}/paymentInstruments"
        method = "GET"
        return self.client.call_adyen_api(
            None, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def get_sweep(self, balanceAccountId, sweepId, idempotency_key=None, **kwargs):
        """
        Get a sweep
        """
        endpoint = self.baseUrl + f"/balanceAccounts/{balanceAccountId}/sweeps/{sweepId}"
        method = "GET"
        return self.client.call_adyen_api(
            None, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def update_balance_account(self, request, id, idempotency_key=None, **kwargs):
        """
        Update a balance account
        """
        endpoint = self.baseUrl + f"/balanceAccounts/{id}"
        method = "PATCH"
        return self.client.call_adyen_api(
            request, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def update_sweep(self, request, balanceAccountId, sweepId, idempotency_key=None, **kwargs):
        """
        Update a sweep
        """
        endpoint = self.baseUrl + f"/balanceAccounts/{balanceAccountId}/sweeps/{sweepId}"
        method = "PATCH"
        return self.client.call_adyen_api(
            request, self.service, method, endpoint, idempotency_key, **kwargs
        )



from ..base import AdyenServiceBase


class PaymentMethodsMerchantLevelApi(AdyenServiceBase):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, client=None):
        super().__init__(client=client)
        self.service = "management"
        self.baseUrl = "https://management-test.adyen.com/v3"

    def add_apple_pay_domain(
        self, request, merchantId, paymentMethodId, idempotency_key=None, **kwargs
    ):
        """
        Add an Apple Pay domain
        """
        endpoint = (
            self.baseUrl
            + f"/merchants/{merchantId}/paymentMethodSettings/{paymentMethodId}/addApplePayDomains"
        )
        method = "POST"
        return self.client.call_adyen_api(
            request, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def get_all_payment_methods(self, merchantId, idempotency_key=None, **kwargs):
        """
        Get all payment methods
        """
        endpoint = self.baseUrl + f"/merchants/{merchantId}/paymentMethodSettings"
        method = "GET"
        return self.client.call_adyen_api(
            None, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def get_apple_pay_domains(self, merchantId, paymentMethodId, idempotency_key=None, **kwargs):
        """
        Get Apple Pay domains
        """
        endpoint = (
            self.baseUrl
            + f"/merchants/{merchantId}/paymentMethodSettings/{paymentMethodId}/getApplePayDomains"
        )
        method = "GET"
        return self.client.call_adyen_api(
            None, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def get_payment_method_details(
        self, merchantId, paymentMethodId, idempotency_key=None, **kwargs
    ):
        """
        Get payment method details
        """
        endpoint = self.baseUrl + f"/merchants/{merchantId}/paymentMethodSettings/{paymentMethodId}"
        method = "GET"
        return self.client.call_adyen_api(
            None, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def request_payment_method(self, request, merchantId, idempotency_key=None, **kwargs):
        """
        Request a payment method
        """
        endpoint = self.baseUrl + f"/merchants/{merchantId}/paymentMethodSettings"
        method = "POST"
        return self.client.call_adyen_api(
            request, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def update_payment_method(
        self, request, merchantId, paymentMethodId, idempotency_key=None, **kwargs
    ):
        """
        Update a payment method
        """
        endpoint = self.baseUrl + f"/merchants/{merchantId}/paymentMethodSettings/{paymentMethodId}"
        method = "PATCH"
        return self.client.call_adyen_api(
            request, self.service, method, endpoint, idempotency_key, **kwargs
        )



from ..base import AdyenServiceBase
from .donations_api import DonationsApi
from .modifications_api import ModificationsApi
from .orders_api import OrdersApi
from .payment_links_api import PaymentLinksApi
from .payments_api import PaymentsApi
from .recurring_api import RecurringApi
from .utility_api import UtilityApi


class AdyenCheckoutApi(AdyenServiceBase):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, client=None):
        super().__init__(client=client)
        self.donations_api = DonationsApi(client=client)
        self.modifications_api = ModificationsApi(client=client)
        self.orders_api = OrdersApi(client=client)
        self.payment_links_api = PaymentLinksApi(client=client)
        self.payments_api = PaymentsApi(client=client)
        self.recurring_api = RecurringApi(client=client)
        self.utility_api = UtilityApi(client=client)



from ..base import AdyenServiceBase


class GrantsApi(AdyenServiceBase):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, client=None):
        super().__init__(client=client)
        self.service = "capital"
        self.baseUrl = "https://balanceplatform-api-test.adyen.com/capital/v1"

    def get_all_grant_disbursements(self, grantId, idempotency_key=None, **kwargs):
        """
        Get all the disbursements of a grant
        """
        endpoint = self.baseUrl + f"/grants/{grantId}/disbursements"
        method = "GET"
        return self.client.call_adyen_api(
            None, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def get_all_grants(self, idempotency_key=None, **kwargs):
        """
        Get all the grants of an account holder
        """
        endpoint = self.baseUrl + "/grants"
        method = "GET"
        return self.client.call_adyen_api(
            None, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def get_grant(self, grantId, idempotency_key=None, **kwargs):
        """
        Get the details of a grant
        """
        endpoint = self.baseUrl + f"/grants/{grantId}"
        method = "GET"
        return self.client.call_adyen_api(
            None, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def get_grant_disbursement(self, grantId, disbursementId, idempotency_key=None, **kwargs):
        """
        Get disbursement details
        """
        endpoint = self.baseUrl + f"/grants/{grantId}/disbursements/{disbursementId}"
        method = "GET"
        return self.client.call_adyen_api(
            None, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def request_grant(self, request, idempotency_key=None, **kwargs):
        """
        Make a request for a grant
        """
        endpoint = self.baseUrl + "/grants"
        method = "POST"
        return self.client.call_adyen_api(
            request, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def update_grant_disbursement(
        self, request, grantId, disbursementId, idempotency_key=None, **kwargs
    ):
        """
        Update the repayment configuration of a disbursement
        """
        endpoint = self.baseUrl + f"/grants/{grantId}/disbursements/{disbursementId}"
        method = "PATCH"
        return self.client.call_adyen_api(
            request, self.service, method, endpoint, idempotency_key, **kwargs
        )



from ..base import AdyenServiceBase


class TransferLimitsBalanceAccountLevelApi(AdyenServiceBase):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, client=None):
        super().__init__(client=client)
        self.service = "balancePlatform"
        self.baseUrl = "https://balanceplatform-api-test.adyen.com/bcl/v2"

    def approve_pending_transfer_limits(self, request, id, idempotency_key=None, **kwargs):
        """
        Approve pending transfer limits
        """
        endpoint = self.baseUrl + f"/balanceAccounts/{id}/transferLimits/approve"
        method = "POST"
        return self.client.call_adyen_api(
            request, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def create_transfer_limit(self, request, id, idempotency_key=None, **kwargs):
        """
        Create a transfer limit
        """
        endpoint = self.baseUrl + f"/balanceAccounts/{id}/transferLimits"
        method = "POST"
        return self.client.call_adyen_api(
            request, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def delete_pending_transfer_limit(self, id, transferLimitId, idempotency_key=None, **kwargs):
        """
        Delete a scheduled or pending transfer limit
        """
        endpoint = self.baseUrl + f"/balanceAccounts/{id}/transferLimits/{transferLimitId}"
        method = "DELETE"
        return self.client.call_adyen_api(
            None, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def get_current_transfer_limits(self, id, idempotency_key=None, **kwargs):
        """
        Get all current transfer limits
        """
        endpoint = self.baseUrl + f"/balanceAccounts/{id}/transferLimits/current"
        method = "GET"
        return self.client.call_adyen_api(
            None, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def get_specific_transfer_limit(self, id, transferLimitId, idempotency_key=None, **kwargs):
        """
        Get the details of a transfer limit
        """
        endpoint = self.baseUrl + f"/balanceAccounts/{id}/transferLimits/{transferLimitId}"
        method = "GET"
        return self.client.call_adyen_api(
            None, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def get_transfer_limits(self, id, idempotency_key=None, **kwargs):
        """
        Filter and view the transfer limits
        """
        endpoint = self.baseUrl + f"/balanceAccounts/{id}/transferLimits"
        method = "GET"
        return self.client.call_adyen_api(
            None, self.service, method, endpoint, idempotency_key, **kwargs
        )



try:
    import requests
except ImportError:
    requests = None

try:
    import pycurl
except ImportError:
    pycurl = None

import base64
import json as json_lib
from io import BytesIO
from urllib.error import HTTPError
from urllib.parse import urlencode
from urllib.request import Request, urlopen


class HTTPClient:
    def __init__(
        self,
        user_agent_suffix,
        lib_version,
        force_request=None,
        timeout=None,
    ):
        # Check if requests already available, default to urllib
        self.user_agent = user_agent_suffix + lib_version
        if not force_request:
            if requests:
                self.request = self._requests_request
            elif pycurl:
                self.request = self._pycurl_request
            else:
                self.request = self._urllib_request
        else:
            if force_request == "requests":
                self.request = self._requests_request
            elif force_request == "pycurl":
                self.request = self._pycurl_request
            else:
                self.request = self._urllib_request

        self.timeout = timeout

    def _pycurl_request(
        self, method, url, json=None, data=None, username="", password="", xapikey="", headers=None
    ):
        """This function will send a request with a specified method to the url endpoint using pycurl.
        Returning an AdyenResult object on 200 HTTP response.
        Either json or data has to be provided for POST/PATCH.
        If username and password are provided, basic auth will be used.
        Args:
            method (str): This is the method used to send the request to an endpoint.
            url (str): url to send the request
            json (dict, optional): Dict of the JSON to POST/PATCH
            data (dict, optional): Dict, presumed flat structure of key/value
                of request to place
            username (str, optionl): Username for basic auth. Must be included
                as part of password.
            password (str, optional): Password for basic auth. Must be included
                as part of username.
            xapikey (str, optional):    Adyen API key.  Will be used for auth
                                        if username and password are absent.
            headers (dict, optional): Key/Value pairs of headers to include
        Returns:
            str:    Raw response received
            str:    Raw request placed
            int:    HTTP status code, eg 200,404,401
            dict:   Key/Value pairs of the headers received.
        """
        if headers is None:
            headers = {}

        response_headers = {}
        stringbuffer = BytesIO()
        curl = pycurl.Curl()
        curl.setopt(curl.URL, url)
        curl.setopt(curl.WRITEDATA, stringbuffer)

        # Add User-Agent header to request so that the
        # request can be identified as coming from the Adyen Python library.
        headers["User-Agent"] = self.user_agent

        if username and password:
            curl.setopt(curl.USERPWD, f"{username}:{password}")
        elif xapikey:
            headers["X-API-KEY"] = xapikey

        # Convert the header dict to formatted array as pycurl needs.
        header_list = [f"{k}:{v}" for k, v in headers.items()]
        # Ensure proper content-type when adding headers
        if json:
            header_list.append("Content-Type:application/json")

        curl.setopt(pycurl.HTTPHEADER, header_list)

        # Return regular dict instead of JSON encoded dict for request:
        if method == "POST" or method == "PATCH":
            raw_store = json

            # Set the request body.
            raw_request = json_lib.dumps(json) if json else urlencode(data)
            curl.setopt(curl.POSTFIELDS, raw_request)
            # Needed here as POSTFIELDS sets the method to POST
            curl.setopt(curl.CUSTOMREQUEST, method)

        elif method == "GET" or method == "DELETE":
            curl.setopt(curl.CUSTOMREQUEST, method)
            raw_store = None

        curl.setopt(curl.TIMEOUT, self.timeout)
        curl.perform()

        # Grab the response content
        result = stringbuffer.getvalue()
        status_code = curl.getinfo(curl.RESPONSE_CODE)
        curl.close()

        # Return regular dict instead of JSON encoded dict for request:
        raw_request = raw_store

        return result, raw_request, status_code, response_headers

    def _requests_request(
        self, method, url, json=None, data=None, username="", password="", xapikey="", headers=None
    ):
        """This function will send a request with a specified method to the url endpoint using requests.
        Returning an AdyenResult object on 200 HTTP response.
        Either json or data has to be provided for POST/PATCH.
        If username and password are provided, basic auth will be used.
        Args:
            method (str): This is the method used to send the request to an endpoint.
            url (str): url to send the request
            json (dict, optional): Dict of the JSON to POST/PATCH
            data (dict, optional): Dict, presumed flat structure of key/value
                of request to place
            username (str, optionl): Username for basic auth. Must be included
                as part of password.
            password (str, optional): Password for basic auth. Must be included
                as part of username.
            xapikey (str, optional):    Adyen API key.  Will be used for auth
                                        if username and password are absent.
            headers (dict, optional): Key/Value pairs of headers to include
        Returns:
            str:    Raw response received
            str:    Raw request placed
            int:    HTTP status code, eg 200,404,401
            dict:   Key/Value pairs of the headers received.
        """
        if headers is None:
            headers = {}

        # Adding basic auth if username and password provided.
        auth = None
        if username and password:
            auth = requests.auth.HTTPBasicAuth(username, password)
        elif xapikey:
            headers["x-api-key"] = xapikey

        # Add User-Agent header to request so that the request
        # can be identified as coming from the Adyen Python library.
        headers["User-Agent"] = self.user_agent
        request = requests.request(
            method=method,
            url=url,
            auth=auth,
            data=data,
            json=json,
            headers=headers,
            timeout=self.timeout,
        )

        # Ensure either json or data is returned for raw request
        # Updated: Only return regular dict,
        # don't switch out formats if this is not important.
        message = json

        return request.text, message, request.status_code, request.headers

    def _urllib_request(
        self,
        method,
        url,
        json=None,
        data=None,
        username="",
        password="",
        xapikey="",
        headers=None,
    ):
        """This function will send a request with a specified method to the url endpoint using urlib2.
        Returning an AdyenResult object on 200 HTTP response.
        Either json or data has to be provided for POST/PATCH.
        If username and password are provided, basic auth will be used.
        Args:
            method (str): This is the method used to send the request to an endpoint.
            url (str): url to send the request
            json (dict, optional): Dict of the JSON to POST/PATCH
            data (dict, optional): Dict, presumed flat structure of key/value
                of request to place
            username (str, optionl): Username for basic auth. Must be included
                as part of password.
            password (str, optional): Password for basic auth. Must be included
                as part of username.
            xapikey (str, optional):    Adyen API key.  Will be used for auth
                                        if username and password are absent.
            headers (dict, optional): Key/Value pairs of headers to include
        Returns:
            str:    Raw response received
            str:    Raw request placed
            int:    HTTP status code, eg 200,404,401
            dict:   Key/Value pairs of the headers received.
        """
        if headers is None:
            headers = {}

        if method == "POST" or method == "PATCH":
            # Store regular dict to return later:
            raw_store = json
            raw_request = json_lib.dumps(json) if json else urlencode(data)
            url_request = Request(url, data=raw_request.encode("utf8"), method=method)
            raw_request = raw_store
            if json:
                url_request.add_header("Content-Type", "application/json")
            elif not data:
                raise ValueError("Please provide either a json or a data field.")

        elif method == "GET" or method == "DELETE":
            url_request = Request(url, method=method)
            raw_request = None

        # Add User-Agent header to request so that the
        # request can be identified as coming from the Adyen Python library.
        headers["User-Agent"] = self.user_agent

        # Set regular dict to return as raw_request:

        # Adding basic auth is username and password provided.
        if username and password:
            basic_authstring = (
                base64.encodebytes(f"{username}:{password}".encode()).decode().replace("\n", "")
            )

            url_request.add_header("Authorization", f"Basic {basic_authstring}")
        elif xapikey:
            headers["X-API-KEY"] = xapikey

        # Adding the headers to the request.
        for key, value in headers.items():
            url_request.add_header(key, str(value))

        # URLlib raises all non 200 responses as en error.
        try:
            response = urlopen(url_request, timeout=self.timeout)
        except HTTPError as e:
            raw_response = e.read()

            return raw_response, raw_request, e.getcode(), e.headers
        else:
            raw_response = response.read()
            response.close()

            # The dict(response.info()) is the headers of the response
            # Raw response, raw request, status code and headers returned
            return (raw_response, raw_request, response.getcode(), dict(response.info()))

    def request(
        self,
        method,
        url,
        json="",
        data="",
        username="",
        password="",
        headers=None,
    ):
        """This is overridden on module initialization. This function will make
        an HTTP method call to a given url. Either json/data will be what is posted to
        the end point. he HTTP request needs to be basicAuth when username and
        password are provided. a headers dict maybe provided,
        whatever the values are should be applied.
        Args:
            url (str):                  url to send the request
            method (str):               method to use for the endpoint
            json (dict, optional):      Dict of the JSON to POST/PATCH
            data (dict, optional):      Dict, presumed flat structure of
                                        key/value of request to place as
                                        www-form
            username (str, optional):    Username for basic auth. Must be
                                        included as part of password.
            password (str, optional):   Password for basic auth. Must be
                                        included as part of username.
            xapikey (str, optional):    Adyen API key.  Will be used for auth
                                        if username and password are absent.
            headers (dict, optional):   Key/Value pairs of headers to include
        Returns:
            str:    Raw request placed
            str:    Raw response received
            int:    HTTP status code, eg 200,404,401
            dict:   Key/Value pairs of the headers received.
        """
        raise NotImplementedError(
            "request of HTTPClient should have been "
            "overridden on initialization. "
            "Otherwise, can be overridden to "
            "supply your own post method"
        )



from ..base import AdyenServiceBase
from .stored_value_api import StoredValueApi


class AdyenStoredValueApi(AdyenServiceBase):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, client=None):
        super().__init__(client=client)
        self.stored_value_api = StoredValueApi(client=client)



from ..base import AdyenServiceBase
from .dynamic_offers_api import DynamicOffersApi
from .grant_accounts_api import GrantAccountsApi
from .grant_offers_api import GrantOffersApi
from .grants_api import GrantsApi


class AdyenCapitalApi(AdyenServiceBase):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, client=None):
        super().__init__(client=client)
        self.dynamic_offers_api = DynamicOffersApi(client=client)
        self.grant_accounts_api = GrantAccountsApi(client=client)
        self.grant_offers_api = GrantOffersApi(client=client)
        self.grants_api = GrantsApi(client=client)



from ..base import AdyenServiceBase


class APIKeyCompanyLevelApi(AdyenServiceBase):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, client=None):
        super().__init__(client=client)
        self.service = "management"
        self.baseUrl = "https://management-test.adyen.com/v3"

    def generate_new_api_key(self, companyId, apiCredentialId, idempotency_key=None, **kwargs):
        """
        Generate new API key
        """
        endpoint = (
            self.baseUrl + f"/companies/{companyId}/apiCredentials/{apiCredentialId}/generateApiKey"
        )
        method = "POST"
        return self.client.call_adyen_api(
            None, self.service, method, endpoint, idempotency_key, **kwargs
        )



from ..base import AdyenServiceBase


class PaymentsAppApi(AdyenServiceBase):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, client=None):
        super().__init__(client=client)
        self.service = "paymentsApp"
        self.baseUrl = "https://management-live.adyen.com/v1"

    def generate_payments_app_boarding_token_for_merchant(
        self, request, merchantId, idempotency_key=None, **kwargs
    ):
        """
        Create a boarding token - merchant level
        """
        endpoint = self.baseUrl + f"/merchants/{merchantId}/generatePaymentsAppBoardingToken"
        method = "POST"
        return self.client.call_adyen_api(
            request, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def generate_payments_app_boarding_token_for_store(
        self, request, merchantId, storeId, idempotency_key=None, **kwargs
    ):
        """
        Create a boarding token - store level
        """
        endpoint = (
            self.baseUrl
            + f"/merchants/{merchantId}/stores/{storeId}/generatePaymentsAppBoardingToken"
        )
        method = "POST"
        return self.client.call_adyen_api(
            request, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def list_payments_app_for_merchant(self, merchantId, idempotency_key=None, **kwargs):
        """
        Get a list of Payments Apps - merchant level
        """
        endpoint = self.baseUrl + f"/merchants/{merchantId}/paymentsApps"
        method = "GET"
        return self.client.call_adyen_api(
            None, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def list_payments_app_for_store(self, merchantId, storeId, idempotency_key=None, **kwargs):
        """
        Get a list of Payments Apps - store level
        """
        endpoint = self.baseUrl + f"/merchants/{merchantId}/stores/{storeId}/paymentsApps"
        method = "GET"
        return self.client.call_adyen_api(
            None, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def revoke_payments_app(self, merchantId, installationId, idempotency_key=None, **kwargs):
        """
        Revoke Payments App instance authentication
        """
        endpoint = self.baseUrl + f"/merchants/{merchantId}/paymentsApps/{installationId}/revoke"
        method = "POST"
        return self.client.call_adyen_api(
            None, self.service, method, endpoint, idempotency_key, **kwargs
        )



from setuptools import find_packages, setup

setup(
    name="Adyen",
    packages=find_packages(include=["Adyen*"], exclude=["tests", "tests.*"]),
    version="15.0.0",
    maintainer="Adyen",
    maintainer_email="support@adyen.com",
    description="Adyen Python Api",
    long_description="A Python client library for accessing Adyen APIs",
    author="Adyen",
    author_email="support@adyen.com",
    url="https://github.com/Adyen/adyen-python-api-library",
    keywords=["payments", "adyen", "fintech"],
    python_requires=">=3.8",
    install_requires=[],
    extras_require={
        "requests": ["requests>=2.25.0"],
        "pycurl": ["pycurl>=7.43.0"],
        "test": [
            "pytest>=7.0.0",
            "pytest-cov>=4.0.0",
            "mock>=4.0.0",
            "requests>=2.25.0",
        ],
        "dev": [
            "ruff>=0.4.4",
            "pre-commit>=3.0.0",
        ],
    },
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Libraries",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Programming Language :: Python :: 3.13",
        "Programming Language :: Python :: 3.14",
    ],
)



from ..base import AdyenServiceBase


class MyAPICredentialApi(AdyenServiceBase):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, client=None):
        super().__init__(client=client)
        self.service = "management"
        self.baseUrl = "https://management-test.adyen.com/v3"

    def add_allowed_origin(self, request, idempotency_key=None, **kwargs):
        """
        Add allowed origin
        """
        endpoint = self.baseUrl + "/me/allowedOrigins"
        method = "POST"
        return self.client.call_adyen_api(
            request, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def generate_client_key(self, idempotency_key=None, **kwargs):
        """
        Generate a client key
        """
        endpoint = self.baseUrl + "/me/generateClientKey"
        method = "POST"
        return self.client.call_adyen_api(
            None, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def get_allowed_origin_details(self, originId, idempotency_key=None, **kwargs):
        """
        Get allowed origin details
        """
        endpoint = self.baseUrl + f"/me/allowedOrigins/{originId}"
        method = "GET"
        return self.client.call_adyen_api(
            None, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def get_allowed_origins(self, idempotency_key=None, **kwargs):
        """
        Get allowed origins
        """
        endpoint = self.baseUrl + "/me/allowedOrigins"
        method = "GET"
        return self.client.call_adyen_api(
            None, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def get_api_credential_details(self, idempotency_key=None, **kwargs):
        """
        Get API credential details
        """
        endpoint = self.baseUrl + "/me"
        method = "GET"
        return self.client.call_adyen_api(
            None, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def remove_allowed_origin(self, originId, idempotency_key=None, **kwargs):
        """
        Remove allowed origin
        """
        endpoint = self.baseUrl + f"/me/allowedOrigins/{originId}"
        method = "DELETE"
        return self.client.call_adyen_api(
            None, self.service, method, endpoint, idempotency_key, **kwargs
        )



from ..base import AdyenServiceBase
from .account_holders_api import AccountHoldersApi
from .authorized_card_users_api import AuthorizedCardUsersApi
from .balance_accounts_api import BalanceAccountsApi
from .balances_api import BalancesApi
from .bank_account_validation_api import BankAccountValidationApi
from .card_orders_api import CardOrdersApi
from .direct_debit_mandates_api import DirectDebitMandatesApi
from .grant_accounts_api import GrantAccountsApi
from .grant_offers_api import GrantOffersApi
from .manage_card_pin_api import ManageCardPINApi
from .manage_sca_devices_api import ManageSCADevicesApi
from .network_tokens_api import NetworkTokensApi
from .payment_instrument_groups_api import PaymentInstrumentGroupsApi
from .payment_instruments_api import PaymentInstrumentsApi
from .platform_api import PlatformApi
from .sca_association_management_api import SCAAssociationManagementApi
from .sca_device_management_api import SCADeviceManagementApi
from .transaction_rules_api import TransactionRulesApi
from .transfer_limits_balance_account_level_api import TransferLimitsBalanceAccountLevelApi
from .transfer_limits_balance_platform_level_api import TransferLimitsBalancePlatformLevelApi
from .transfer_routes_api import TransferRoutesApi


class AdyenBalancePlatformApi(AdyenServiceBase):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, client=None):
        super().__init__(client=client)
        self.account_holders_api = AccountHoldersApi(client=client)
        self.authorized_card_users_api = AuthorizedCardUsersApi(client=client)
        self.balance_accounts_api = BalanceAccountsApi(client=client)
        self.balances_api = BalancesApi(client=client)
        self.bank_account_validation_api = BankAccountValidationApi(client=client)
        self.card_orders_api = CardOrdersApi(client=client)
        self.direct_debit_mandates_api = DirectDebitMandatesApi(client=client)
        self.grant_accounts_api = GrantAccountsApi(client=client)
        self.grant_offers_api = GrantOffersApi(client=client)
        self.manage_sca_devices_api = ManageSCADevicesApi(client=client)
        self.manage_card_pin_api = ManageCardPINApi(client=client)
        self.network_tokens_api = NetworkTokensApi(client=client)
        self.payment_instrument_groups_api = PaymentInstrumentGroupsApi(client=client)
        self.payment_instruments_api = PaymentInstrumentsApi(client=client)
        self.platform_api = PlatformApi(client=client)
        self.sca_association_management_api = SCAAssociationManagementApi(client=client)
        self.sca_device_management_api = SCADeviceManagementApi(client=client)
        self.transaction_rules_api = TransactionRulesApi(client=client)
        self.transfer_limits_balance_account_level_api = TransferLimitsBalanceAccountLevelApi(
            client=client
        )
        self.transfer_limits_balance_platform_level_api = TransferLimitsBalancePlatformLevelApi(
            client=client
        )
        self.transfer_routes_api = TransferRoutesApi(client=client)



from __future__ import annotations

import json as json_lib
import re
from typing import Any

from . import settings, util
from .exceptions import (
    AdyenAPIAuthenticationError,
    AdyenAPICommunicationError,
    AdyenAPIInvalidPermission,
    AdyenAPIResponseError,
    AdyenAPIUnprocessableEntity,
    AdyenAPIValidationError,
    AdyenEndpointInvalidFormat,
    AdyenInvalidRequestError,
)
from .httpclient import HTTPClient


class AdyenResult:
    """
    Args:
        message (dict, optional): Parsed message returned from API client.
        status_code (int, optional): Default 200. HTTP response code, ie 200,
            404, 500, etc.
        psp (str, optional): Psp reference returned by Adyen for a payment.
        raw_request (str, optional): Raw request placed to Adyen.
        raw_response (str, optional): Raw response returned by Adyen.

    """

    def __init__(
        self,
        message: dict[str, Any] | None = None,
        status_code: int = 200,
        psp: str = "",
        raw_request: str = "",
        raw_response: str = "",
    ) -> None:
        self.message = message
        self.status_code = status_code
        self.psp = psp
        self.raw_request = raw_request
        self.raw_response = raw_response
        self.details: dict[str, Any] = {}

    def __str__(self) -> str:
        return repr(self.message)


class AdyenClient:
    IDEMPOTENCY_HEADER_NAME = "Idempotency-Key"
    APPLICATION_INFO_HEADER_NAME = "adyen-library-name"
    APPLICATION_VERSION_HEADER_NAME = "adyen-library-version"
    """A requesting client that interacts with Adyen. This class holds the
    adyen logic of Adyen HTTP API communication. This is the object that can
    maintain its own username, password and hmac.
    When these values aren't within this object, the root adyen module
    variables will be used.

    The public methods and call_api only return AdyenResult objects.
    Otherwise raising various validation and communication errors.

    Args:
        username (str, optional): Username of webservice user
        password (str, optional): Password of webservice user
        merchant_account (str, optional): Merchant account for requests to be
            placed through
        platform (str, optional): Defaults "test". The Adyen platform to make
            requests against.
        hmac (str, optional): Hmac key that is used for signature calculation.
        http_timeout (int, optional): The timeout in seconds for HTTP calls,
            default 30.
    """

    def __init__(
        self,
        username: str | None = None,
        password: str | None = None,
        xapikey: str | None = None,
        application_name: str | None = None,
        review_payout_username: str | None = None,
        review_payout_password: str | None = None,
        store_payout_username: str | None = None,
        store_payout_password: str | None = None,
        platform: str = "test",
        merchant_account: str | None = None,
        merchant_specific_url: str | None = None,
        hmac: str | None = None,
        http_force: str | None = None,
        live_endpoint_prefix: str | None = None,
        http_timeout: int = 30,
        api_bin_lookup_version: str | int | None = None,
        api_checkout_version: str | int | None = None,
        api_management_version: str | int | None = None,
        api_payment_version: str | int | None = None,
        api_payout_version: str | int | None = None,
        api_recurring_version: str | int | None = None,
        api_terminal_version: str | int | None = None,
        api_legal_entity_management_version: str | int | None = None,
        api_data_protection_version: str | int | None = None,
        api_transfers_version: str | int | None = None,
        api_stored_value_version: str | int | None = None,
        api_balance_platform_version: str | int | None = None,
        api_disputes_version: str | int | None = None,
        api_session_authentication_version: str | int | None = None,
        api_capital_version: str | int | None = None,
    ) -> None:
        self.username = username
        self.password = password
        self.xapikey = xapikey
        self.application_name = application_name
        self.review_payout_username = review_payout_username
        self.review_payout_password = review_payout_password
        self.store_payout_username = store_payout_username
        self.store_payout_password = store_payout_password
        self.platform = platform
        self.merchant_specific_url = merchant_specific_url
        self.hmac = hmac
        self.merchant_account = merchant_account
        self.psp_list: list[str] = []
        self.LIB_VERSION = settings.LIB_VERSION
        self.USER_AGENT_SUFFIX = settings.LIB_NAME + "/"
        self.http_init = False
        self.http_force = http_force
        self.live_endpoint_prefix = live_endpoint_prefix
        self.http_timeout = http_timeout
        self.api_bin_lookup_version = api_bin_lookup_version
        self.api_checkout_version = api_checkout_version
        self.api_management_version = api_management_version
        self.api_payment_version = api_payment_version
        self.api_payout_version = api_payout_version
        self.api_recurring_version = api_recurring_version
        self.api_terminal_version = api_terminal_version
        self.api_legal_entity_management_version = api_legal_entity_management_version
        self.api_data_protection_version = api_data_protection_version
        self.api_transfers_version = api_transfers_version
        self.api_stored_value_version = api_stored_value_version
        self.api_balance_platform_version = api_balance_platform_version
        self.api_disputes_version = api_disputes_version
        self.api_session_authentication_version = api_session_authentication_version
        self.api_capital_version = api_capital_version

    def _require_live_endpoint_prefix(self):
        if self.live_endpoint_prefix is None:
            error_string = (
                "Please set your live suffix. You can set it by running "
                "adyen.client.live_endpoint_prefix = 'Your live suffix'"
            )
            raise AdyenEndpointInvalidFormat(error_string)
        return self.live_endpoint_prefix

    def _determine_api_url(self, platform, endpoint):
        if platform == "test":
            # Replace live with test in base url is configured with live url by default
            return endpoint.replace("-live", "-test")

        if "pal-" in endpoint:
            live_endpoint_prefix = self._require_live_endpoint_prefix()
            endpoint = endpoint.replace(
                "https://pal-test.adyen.com/pal/servlet/",
                f"https://{live_endpoint_prefix}-pal-live.adyenpayments.com/pal/servlet/",
            )
        elif "paltokenization-" in endpoint:
            live_endpoint_prefix = self._require_live_endpoint_prefix()
            endpoint = endpoint.replace(
                "https://paltokenization-test.adyen.com/paltokenization/servlet/",
                f"https://{live_endpoint_prefix}-paltokenization-live.adyenpayments.com/paltokenization/servlet/",
            )
        elif "checkout-" in endpoint:
            live_endpoint_prefix = self._require_live_endpoint_prefix()

            if "possdk" in endpoint:
                endpoint = endpoint.replace(
                    "https://checkout-test.adyen.com/",
                    f"https://{live_endpoint_prefix}-checkout-live.adyenpayments.com/",
                )
            else:
                endpoint = endpoint.replace(
                    "https://checkout-test.adyen.com/",
                    f"https://{live_endpoint_prefix}-checkout-live.adyenpayments.com/checkout/",
                )
        elif "authe/api" in endpoint:
            endpoint = endpoint.replace("https://test.adyen.com", "https://authe-live.adyen.com")

        endpoint = endpoint.replace("-test", "-live")

        return endpoint

    def _review_payout_username(self, **kwargs):
        if "username" in kwargs:
            return kwargs["username"]
        elif self.review_payout_username:
            return self.review_payout_username
        errorstring = (
            "Please set your review payout "
            "webservice username. You can do this by running "
            "Adyen.review_payout_username = 'Your payout username'"
        )
        raise AdyenInvalidRequestError(errorstring)

    def _review_payout_pass(self, **kwargs):
        if "password" in kwargs:
            return kwargs["password"]
        elif self.review_payout_password:
            return self.review_payout_password
        errorstring = (
            "Please set your review payout "
            "webservice password. You can do this by running "
            "Adyen.review_payout_password = 'Your payout password"
        )
        raise AdyenInvalidRequestError(errorstring)

    def _store_payout_username(self, **kwargs):
        if "username" in kwargs:
            return kwargs["username"]
        elif self.store_payout_username:
            return self.store_payout_username
        errorstring = (
            "Please set your review payout "
            "webservice username. You can do this by running "
            "Adyen.review_payout_username = 'Your payout username'"
        )
        raise AdyenInvalidRequestError(errorstring)

    def _store_payout_pass(self, **kwargs):
        if "password" in kwargs:
            return kwargs["password"]
        elif self.store_payout_password:
            return self.store_payout_password
        errorstring = (
            "Please set your review payout "
            "webservice password. You can do this by running "
            "Adyen.review_payout_password = 'Your payout password"
        )
        raise AdyenInvalidRequestError(errorstring)

    def _set_credentials(self, service, endpoint, **kwargs):
        xapikey = None
        username = None
        password = None

        # username at self object has highest priority. fallback to root module
        # and ensure that it is set.

        if self.xapikey:
            xapikey = self.xapikey
        elif "xapikey" in kwargs:
            xapikey = kwargs.pop("xapikey")

        if self.username:
            username = self.username
        elif "username" in kwargs:
            username = kwargs.pop("username")
        if service == "Payout":
            if any(substring in endpoint for substring in ["store", "submit"]):
                username = self._store_payout_username(**kwargs)
            else:
                username = self._review_payout_username(**kwargs)

        if not username and not xapikey:
            errorstring = (
                "Please set your webservice username.You can do this by running "
                "Adyen.username = 'Your username'"
            )
            raise AdyenInvalidRequestError(errorstring)
            # password at self object has highest priority.
            # fallback to root module
            # and ensure that it is set.

        if self.password and not xapikey:
            password = self.password
        elif "password" in kwargs:
            password = kwargs.pop("password")
        if service == "Payout":
            if any(substring in endpoint for substring in ["store", "submit"]):
                password = self._store_payout_pass(**kwargs)
            else:
                password = self._review_payout_pass(**kwargs)

        if not password and not xapikey:
            errorstring = (
                "Please set your webservice password.You can do this by running "
                "Adyen.password = 'Your password'"
            )
            raise AdyenInvalidRequestError(errorstring)
            # xapikey at self object has highest priority.
            # fallback to root module
            # and ensure that it is set.

        return xapikey, username, password, kwargs

    def _set_platform(self, **kwargs):
        # platform at self object has highest priority. fallback to root module
        # and ensure that it is set to either 'live' or 'test'.
        platform = None
        if self.platform:
            platform = self.platform
        elif "platform" in kwargs:
            platform = kwargs.pop("platform")

        if not isinstance(platform, str):
            errorstring = "'platform' value must be type of string"
            raise TypeError(errorstring)
        elif platform.lower() not in ["live", "test"]:
            errorstring = "'platform' must be the value of 'live' or 'test'"
            raise ValueError(errorstring)

        return platform

    def _set_url_version(self, service, endpoint):
        version_lookup = {
            "binlookup": self.api_bin_lookup_version,
            "checkout": self.api_checkout_version,
            "management": self.api_management_version,
            "payments": self.api_payment_version,
            "payouts": self.api_payout_version,
            "recurring": self.api_recurring_version,
            "terminal": self.api_terminal_version,
            "legalEntityManagement": self.api_legal_entity_management_version,
            "dataProtection": self.api_data_protection_version,
            "transfers": self.api_transfers_version,
            "storedValue": self.api_stored_value_version,
            "balancePlatform": self.api_balance_platform_version,
            "disputes": self.api_disputes_version,
            "sessionAuthentication": self.api_session_authentication_version,
            "capital": self.api_capital_version,
        }

        new_version = f"v{version_lookup[service]}"
        endpoint = re.sub(r"\.com/v\d{1,2}", f".com/{new_version}", endpoint)
        return endpoint

    def call_adyen_api(
        self, request_data, service, method, endpoint, idempotency_key=None, **kwargs
    ):
        """This will call the adyen api. username, password,
        and platform are pulled from root module level and or self object.
        AdyenResult will be returned on 200 response. Otherwise, an exception
        is raised.

        Args:
            idempotency_key: https://docs.adyen.com/development-resources
            /api-idempotency
            request_data (dict): The dictionary of the request to place. This
                should be in the structure of the Adyen API.
                https://docs.adyen.com/api-explorer
            service (str): This is the API service to be called.
            method (str): This is the method used to send the request to an endpoint.
            endpoint (str): The specific endpoint of the API service to be called
        Returns:
            AdyenResult: The AdyenResult is returned when a request was
                successful.
        """
        # Initialize http client
        if not self.http_init:
            self._init_http_client()

        # Set credentials
        xapikey, username, password, kwargs = self._set_credentials(service, endpoint, **kwargs)
        # Set platform
        platform = self._set_platform(**kwargs)
        message = request_data
        # Set version (if not default one)
        versions = [
            self.api_bin_lookup_version,
            self.api_checkout_version,
            self.api_management_version,
            self.api_payment_version,
            self.api_payout_version,
            self.api_recurring_version,
            self.api_terminal_version,
            self.api_legal_entity_management_version,
            self.api_data_protection_version,
            self.api_transfers_version,
            self.api_stored_value_version,
            self.api_balance_platform_version,
            self.api_disputes_version,
            self.api_session_authentication_version,
            self.api_capital_version,
        ]
        if any(versions):
            endpoint = self._set_url_version(service, endpoint)

        headers = {
            self.APPLICATION_INFO_HEADER_NAME: settings.LIB_NAME,
            self.APPLICATION_VERSION_HEADER_NAME: settings.LIB_VERSION,
            "User-Agent": self.http_client.user_agent,
        }

        # Adyen requires this header to be set and uses the combination of
        # merchant account and merchant reference to determine uniqueness.
        if idempotency_key:
            headers[self.IDEMPOTENCY_HEADER_NAME] = idempotency_key

        # Additional headers provided via the `header_parameters` keyword argument
        # Pass as a keyword argument in the method call.
        if "header_parameters" in kwargs:
            headers.update(kwargs["header_parameters"])
            kwargs.pop("header_parameters")

        url = self._determine_api_url(platform, endpoint)

        if "query_parameters" in kwargs:
            url = url + util.get_query(kwargs["query_parameters"])
            kwargs.pop("query_parameters")

        if xapikey:
            raw_response, raw_request, status_code, headers = self.http_client.request(
                method, url, json=request_data, xapikey=xapikey, headers=headers, **kwargs
            )
        else:
            raw_response, raw_request, status_code, headers = self.http_client.request(
                method,
                url,
                json=message,
                username=username,
                password=password,
                headers=headers,
                **kwargs,
            )

        # Creates AdyenResponse if request was successful, raises error if not.
        adyen_result = self._handle_response(url, raw_response, raw_request, status_code, headers)

        return adyen_result

    def _init_http_client(self):
        user_agent_suffix = self.USER_AGENT_SUFFIX
        if self.application_name:
            user_agent_suffix = self.application_name + " " + user_agent_suffix
        self.http_client = HTTPClient(
            user_agent_suffix=user_agent_suffix,
            lib_version=self.LIB_VERSION,
            force_request=self.http_force,
            timeout=self.http_timeout,
        )
        self.http_init = True

    def _handle_response(self, url, raw_response, raw_request, status_code, headers):
        """This parses the content from raw communication, raising an error if
        anything other than 2xx was returned.

        Args:
            url (str): URL where request was made
            raw_response (str): The raw communication sent to Adyen
            raw_request (str): The raw response returned by Adyen
            status_code (int): The HTTP status code
            headers (dict): Key/Value of the headers.

        Returns:
            AdyenResult: Result object if successful.
        """

        try:
            response = json_lib.loads(raw_response)
        except json_lib.JSONDecodeError:
            response = {}

        if status_code not in [200, 201, 202, 204]:
            self._raise_http_error(
                url,
                response,
                status_code,
                headers.get("pspReference"),
                raw_request,
                raw_response,
                headers,
            )
        else:
            psp = self._get_psp(response, headers)
            return AdyenResult(
                message=response,
                status_code=status_code,
                psp=psp,
                raw_request=raw_request,
                raw_response=raw_response,
            )

    @staticmethod
    def _raise_http_error(
        url, response_obj, status_code, psp_ref, raw_request, raw_response, headers
    ):
        """This function handles the non 2xx responses from Adyen, raising an
        error that should provide more information.

        Args:
            url (str): url of the request
            response_obj (dict): Dict containing the parsed JSON response from
                Adyen
            status_code (int): HTTP status code of the request
            psp_ref (str): Psp reference of the request attempt
            raw_request (str): The raw request placed to Adyen
            raw_response (str): The raw response(body) returned by Adyen
            headers(dict): headers of the response

        Returns:
            None: It never returns
        """

        if response_obj == {}:
            message = raw_response
        else:
            message = response_obj

        error_code = response_obj.get("errorCode")

        if status_code == 404:
            raise AdyenAPICommunicationError(
                message, raw_request, raw_response, url, psp_ref, headers, status_code, error_code
            )
        elif status_code == 400:
            raise AdyenAPIValidationError(
                message, raw_request, raw_response, url, psp_ref, headers, status_code, error_code
            )
        elif status_code == 401:
            raise AdyenAPIAuthenticationError(
                message, raw_request, raw_response, url, psp_ref, headers, status_code, error_code
            )
        elif status_code == 403:
            raise AdyenAPIInvalidPermission(
                message, raw_request, raw_response, url, psp_ref, headers, status_code, error_code
            )
        elif status_code == 422:
            raise AdyenAPIUnprocessableEntity(
                message, raw_request, raw_response, url, psp_ref, headers, status_code, error_code
            )
        elif status_code == 500:
            raise AdyenAPICommunicationError(
                message, raw_request, raw_response, url, psp_ref, headers, status_code, error_code
            )

        raise AdyenAPIResponseError(
            message, raw_request, raw_response, url, psp_ref, headers, status_code, error_code
        )

    @staticmethod
    def _get_psp(response, headers):
        psp_ref = response.get("pspReference")
        if psp_ref is None:
            psp_ref = headers.get("pspReference")
        return psp_ref



from ..base import AdyenServiceBase


class GrantAccountsApi(AdyenServiceBase):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, client=None):
        super().__init__(client=client)
        self.service = "capital"
        self.baseUrl = "https://balanceplatform-api-test.adyen.com/capital/v1"

    def get_grant_account_information(self, id, idempotency_key=None, **kwargs):
        """
        Get the information of your grant account
        """
        endpoint = self.baseUrl + f"/grantAccounts/{id}"
        method = "GET"
        return self.client.call_adyen_api(
            None, self.service, method, endpoint, idempotency_key, **kwargs
        )



from ..base import AdyenServiceBase


class ModificationsApi(AdyenServiceBase):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, client=None):
        super().__init__(client=client)
        self.service = "checkout"
        self.baseUrl = "https://checkout-test.adyen.com/v71"

    def cancel_authorised_payment(self, request, idempotency_key=None, **kwargs):
        """
        Cancel an authorised payment
        """
        endpoint = self.baseUrl + "/cancels"
        method = "POST"
        return self.client.call_adyen_api(
            request, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def cancel_authorised_payment_by_psp_reference(
        self, request, paymentPspReference, idempotency_key=None, **kwargs
    ):
        """
        Cancel an authorised payment
        """
        endpoint = self.baseUrl + f"/payments/{paymentPspReference}/cancels"
        method = "POST"
        return self.client.call_adyen_api(
            request, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def capture_authorised_payment(
        self, request, paymentPspReference, idempotency_key=None, **kwargs
    ):
        """
        Capture an authorised payment
        """
        endpoint = self.baseUrl + f"/payments/{paymentPspReference}/captures"
        method = "POST"
        return self.client.call_adyen_api(
            request, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def refund_captured_payment(self, request, paymentPspReference, idempotency_key=None, **kwargs):
        """
        Refund a captured payment
        """
        endpoint = self.baseUrl + f"/payments/{paymentPspReference}/refunds"
        method = "POST"
        return self.client.call_adyen_api(
            request, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def refund_or_cancel_payment(
        self, request, paymentPspReference, idempotency_key=None, **kwargs
    ):
        """
        Refund or cancel a payment
        """
        endpoint = self.baseUrl + f"/payments/{paymentPspReference}/reversals"
        method = "POST"
        return self.client.call_adyen_api(
            request, self.service, method, endpoint, idempotency_key, **kwargs
        )

    def update_authorised_amount(
        self, request, paymentPspReference, idempotency_key=None, **kwargs
    ):
        """
        Update an authorised amount
        """
        endpoint = self.baseUrl + f"/payments/{paymentPspReference}/amountUpdates"
        method = "POST"
        return self.client.call_adyen_api(
            request, self.service, method, endpoint, idempotency_key, **kwargs
        )
